@@ -30,17 +30,16 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <tiff.h>
-#include <tiffio.h>
-#include <gconf/gconf-client.h>
 
 #include "gcm-calibrate.h"
+#include "gcm-xyz.h"
 #include "gcm-dmi.h"
 #include "gcm-device-xrandr.h"
 #include "gcm-utils.h"
 #include "gcm-brightness.h"
 #include "gcm-colorimeter.h"
 #include "gcm-calibrate-dialog.h"
+#include "gcm-exif.h"
 
 #include "egg-debug.h"
 
@@ -64,6 +63,7 @@ struct _GcmCalibratePrivate
 	GcmColorimeterKind		 colorimeter_kind;
 	GcmCalibrateDialog		*calibrate_dialog;
 	GcmDeviceKind			 device_kind;
+	GcmXyz				*xyz;
 	gchar				*output_name;
 	gchar				*filename_source;
 	gchar				*filename_reference;
@@ -75,7 +75,7 @@ struct _GcmCalibratePrivate
 	gchar				*serial;
 	gchar				*device;
 	gchar				*working_path;
-	GConfClient			*gconf_client;
+	GSettings			*settings;
 };
 
 enum {
@@ -97,29 +97,12 @@ enum {
 	PROP_FILENAME_RESULT,
 	PROP_WORKING_PATH,
 	PROP_PRECISION,
+	PROP_XYZ,
 	PROP_LAST
 };
 
 G_DEFINE_TYPE (GcmCalibrate, gcm_calibrate, G_TYPE_OBJECT)
 
-/**
- * gcm_calibrate_precision_from_string:
- **/
-static GcmCalibratePrecision
-gcm_calibrate_precision_from_string (const gchar *string)
-{
-	if (g_strcmp0 (string, "short") == 0)
-		return GCM_CALIBRATE_PRECISION_SHORT;
-	if (g_strcmp0 (string, "normal") == 0)
-		return GCM_CALIBRATE_PRECISION_NORMAL;
-	if (g_strcmp0 (string, "long") == 0)
-		return GCM_CALIBRATE_PRECISION_LONG;
-	if (g_strcmp0 (string, "ask") == 0)
-		return GCM_CALIBRATE_PRECISION_UNKNOWN;
-	egg_warning ("failed to convert to precision: %s", string);
-	return GCM_CALIBRATE_PRECISION_UNKNOWN;
-}
-
 /**
  * gcm_calibrate_get_model_fallback:
  **/
@@ -203,6 +186,15 @@ gcm_calibrate_get_filename_result (GcmCalibrate *calibrate)
 	return calibrate->priv->filename_result;
 }
 
+/**
+ * gcm_calibrate_get_working_path:
+ **/
+const gchar *
+gcm_calibrate_get_working_path (GcmCalibrate *calibrate)
+{
+	return calibrate->priv->working_path;
+}
+
 /**
  * gcm_calibrate_set_basename:
  **/
@@ -238,7 +230,7 @@ gcm_calibrate_set_basename (GcmCalibrate *calibrate)
 		g_string_append_printf (basename, " - %s", serial);
 	g_string_append_printf (basename, " (%04i-%02i-%02i)", date->year, date->month, date->day);
 
-	/* maybe configure in GConf? */
+	/* maybe configure in GSettings? */
 	if (0)
 		g_string_append_printf (basename, " [%s]", timespec);
 
@@ -324,28 +316,25 @@ out:
 gboolean
 gcm_calibrate_set_from_exif (GcmCalibrate *calibrate, const gchar *filename, GError **error)
 {
-	const gchar *manufacturer = NULL;
-	const gchar *model = NULL;
+	const gchar *manufacturer;
+	const gchar *model;
+	const gchar *serial;
 	gchar *description = NULL;
-	const gchar *serial = NULL;
-	TIFF *tiff;
-	gboolean ret = TRUE;
+	gboolean ret;
+	GcmExif *exif;
+	GFile *file;
 
-	/* open file */
-	tiff = TIFFOpen (filename, "r");
-	TIFFGetField (tiff,TIFFTAG_MAKE, &manufacturer);
-	TIFFGetField (tiff,TIFFTAG_MODEL, &model);
-	TIFFGetField (tiff,TIFFTAG_CAMERASERIALNUMBER, &serial);
-
-	/* we failed to get data */
-	if (manufacturer == NULL || model == NULL) {
-		g_set_error (error,
-			     GCM_CALIBRATE_ERROR,
-			     GCM_CALIBRATE_ERROR_NO_DATA,
-			     "failed to get EXIF data from TIFF");
-		ret = FALSE;
+	/* parse file */
+	exif = gcm_exif_new ();
+	file = g_file_new_for_path (filename);
+	ret = gcm_exif_parse (exif, file, error);
+	if (!ret)
 		goto out;
-	}
+
+	/* get data */
+	manufacturer = gcm_exif_get_manufacturer (exif);
+	model = gcm_exif_get_model (exif);
+	serial = gcm_exif_get_serial (exif);
 
 	/* do the best we can */
 	description = g_strdup_printf ("%s - %s", manufacturer, model);
@@ -361,8 +350,9 @@ gcm_calibrate_set_from_exif (GcmCalibrate *calibrate, const gchar *filename, GEr
 		g_object_set (calibrate, "serial", serial, NULL);
 
 out:
+	g_object_unref (file);
+	g_object_unref (exif);
 	g_free (description);
-	TIFFClose (tiff);
 	return ret;
 }
 
@@ -471,27 +461,18 @@ gcm_calibrate_get_precision (GcmCalibrate *calibrate, GError **error)
 	string = g_string_new ("");
 
 	/* TRANSLATORS: dialog title */
-	title = _("Choose the precision of the profile");
-
-	/* TRANSLATORS: dialog message, suffix */
-	g_string_append_printf (string, "%s\n", _("Please choose the profile precision."));
+	title = _("Profile Precision");
 
 	/* TRANSLATORS: this is the message body for the chart selection */
-	g_string_append_printf (string, "\n\n%s", _("For a typical workflow, a normal precision profile is sufficient."));
+	g_string_append (string, _("A higher precision profile provides higher accuracy in color matching but requires more time for reading the color patches."));
 
 	/* TRANSLATORS: this is the message body for the chart selection */
-	g_string_append_printf (string, "\n%s", _("High precision profiles provide higher accuracy in color matching. Correspondingly, low precision profiles result in lower quality."));
+	g_string_append_printf (string, "\n%s", _("For a typical workflow, a normal precision profile is sufficient."));
 
 	/* printer specific options */
 	if (priv->device_kind == GCM_DEVICE_KIND_PRINTER) {
 		/* TRANSLATORS: dialog message, preface */
-		g_string_append_printf (string, "\n%s", _("The high precision profiles also require more paper and time for reading the color swatches."));
-	}
-
-	/* display specific options */
-	if (priv->device_kind == GCM_DEVICE_KIND_DISPLAY) {
-		/* TRANSLATORS: dialog message, preface */
-		g_string_append_printf (string, "\n%s", _("The high precision profiles also require more time for reading the color swatches."));
+		g_string_append_printf (string, "\n%s", _("The high precision profile also requires more paper and printer ink."));
 	}
 
 	/* push new messages into the UI */
@@ -504,7 +485,7 @@ gcm_calibrate_get_precision (GcmCalibrate *calibrate, GError **error)
 		g_set_error_literal (error,
 				     GCM_CALIBRATE_ERROR,
 				     GCM_CALIBRATE_ERROR_USER_ABORT,
-				     "user did not choose precision type and ask is specified in GConf");
+				     "user did not choose precision type and ask is specified in GSettings");
 		goto out;
 	}
 
@@ -516,6 +497,36 @@ out:
 	return precision;
 }
 
+/**
+ * gcm_calibrate_spotread:
+ **/
+gboolean
+gcm_calibrate_spotread (GcmCalibrate *calibrate, GtkWindow *window, GError **error)
+{
+	GcmCalibrateClass *klass = GCM_CALIBRATE_GET_CLASS (calibrate);
+	gboolean ret;
+
+	/* set the per-profile filename */
+	ret = gcm_calibrate_set_working_path (calibrate, error);
+	if (!ret)
+		goto out;
+
+	/* coldplug source */
+	if (klass->calibrate_spotread == NULL) {
+		ret = FALSE;
+		g_set_error_literal (error,
+				     GCM_CALIBRATE_ERROR,
+				     GCM_CALIBRATE_ERROR_INTERNAL,
+				     "no klass support");
+		goto out;
+	}
+
+	/* proxy */
+	ret = klass->calibrate_spotread (calibrate, window, error);
+out:
+	return ret;
+}
+
 /**
  * gcm_calibrate_display:
  **/
@@ -581,8 +592,7 @@ gcm_calibrate_display (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	}
 
 	/* get default precision */
-	precision = gconf_client_get_string (priv->gconf_client, GCM_SETTINGS_CALIBRATION_LENGTH, NULL);
-	priv->precision = gcm_calibrate_precision_from_string (precision);
+	priv->precision = g_settings_get_enum (priv->settings, GCM_SETTINGS_CALIBRATION_LENGTH);
 	if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
 		priv->precision = gcm_calibrate_get_precision (calibrate, error);
 		if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
@@ -696,8 +706,10 @@ gcm_calibrate_device_get_reference_image (const gchar *directory, GtkWindow *win
 	GtkWidget *dialog;
 	GtkFileFilter *filter;
 
-	/* TRANSLATORS: dialog for file->open dialog */
-	dialog = gtk_file_chooser_dialog_new (_("Select reference image"), window,
+	/* TRANSLATORS: dialog for file->open dialog. A calibration target image is the
+	 * aquired image of the calibration target, e.g. an image file that looks
+	 * a bit like this: http://www.colorreference.de/targets/target.jpg */
+	dialog = gtk_file_chooser_dialog_new (_("Select calibration target image"), window,
 					       GTK_FILE_CHOOSER_ACTION_OPEN,
 					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					       GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
@@ -876,8 +888,10 @@ gcm_calibrate_printer (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	/* TRANSLATORS: title, you can profile all at once, or in steps */
 	title = _("Please choose a profiling mode");
 
-	/* TRANSLATORS: dialog message */
-	message = _("Please indicate if you want to profile a local printer, generate some reference images, or process some reference images.");
+	/* TRANSLATORS: dialog message. Test patches are pages of colored squares
+	 * that are printed with a printer, and then read in with a calibration
+	 * device to create a profile */
+	message = _("Please indicate if you want to profile a local printer, generate some test patches, or profile using existing test patches.");
 
 	/* push new messages into the UI */
 	gcm_calibrate_dialog_show (priv->calibrate_dialog, GCM_CALIBRATE_DIALOG_TAB_PRINT_MODE, title, message);
@@ -895,8 +909,7 @@ gcm_calibrate_printer (GcmCalibrate *calibrate, GtkWindow *window, GError **erro
 	}
 
 	/* get default precision */
-	precision = gconf_client_get_string (priv->gconf_client, GCM_SETTINGS_CALIBRATION_LENGTH, NULL);
-	priv->precision = gcm_calibrate_precision_from_string (precision);
+	priv->precision = g_settings_get_enum (priv->settings, GCM_SETTINGS_CALIBRATION_LENGTH);
 	if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
 		priv->precision = gcm_calibrate_get_precision (calibrate, error);
 		if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
@@ -973,7 +986,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	GtkResponseType response;
 	GtkWindow *window_tmp;
 	gchar *precision = NULL;
-#ifdef GCM_USE_PACKAGEKIT
+#ifdef HAVE_PACKAGEKIT
 	GtkWidget *dialog;
 #endif
 	const gchar *title;
@@ -985,7 +998,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	/* install shared-color-targets package */
 	has_shared_targets = g_file_test ("/usr/share/shared-color-targets", G_FILE_TEST_IS_DIR);
 	if (!has_shared_targets) {
-#ifdef GCM_USE_PACKAGEKIT
+#ifdef HAVE_PACKAGEKIT
 		/* ask the user to confirm */
 		dialog = gtk_message_dialog_new (window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
 						 /* TRANSLATORS: title, usually we can tell based on the EDID data or output name */
@@ -1017,13 +1030,15 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 #endif
 	}
 
-	/* TRANSLATORS: this is the window title for when the user selects the chart type.
-	                A chart is a type of reference image the user has purchased. */
-	title = _("Please select chart type");
+	/* TRANSLATORS: this is the window title for when the user selects the calibration target.
+	 * A calibration target is an accuratly printed grid of colors, for instance:
+	 * the IT 8.7 targets available here: http://www.targets.coloraid.de/ */
+	title = _("Please select a calibration target");
 	g_string_set_size (string, 0);
 
-	/* TRANSLATORS: dialog message, preface */
-	g_string_append_printf (string, "%s\n", _("Before profiling the device, you have to manually capture an image of a calibrated target and save it as a TIFF image file."));
+	/* TRANSLATORS: dialog message, preface. A calibration target looks like
+	 * this: http://www.colorreference.de/targets/target.jpg */
+	g_string_append_printf (string, "%s\n", _("Before profiling the device, you have to manually capture an image of a calibration target and save it as a TIFF image file."));
 
 	/* scanner specific options */
 	if (priv->device_kind == GCM_DEVICE_KIND_SCANNER) {
@@ -1044,7 +1059,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	g_string_append_printf (string, "\n%s\n", _("For best results, the reference target should also be less than two years old."));
 
 	/* TRANSLATORS: this is the message body for the chart selection */
-	g_string_append_printf (string, "\n%s\n", _("Please select the chart type which corresponds to your reference file."));
+	g_string_append_printf (string, "\n%s\n", _("Please select the calibration target type which corresponds to your reference file."));
 
 	/* push new messages into the UI */
 	gcm_calibrate_dialog_set_window (priv->calibrate_dialog, window);
@@ -1057,7 +1072,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 		g_set_error_literal (error,
 				     GCM_CALIBRATE_ERROR,
 				     GCM_CALIBRATE_ERROR_USER_ABORT,
-				     "user did not choose chart type");
+				     "user did not choose calibration target type");
 		ret = FALSE;
 		goto out;
 	}
@@ -1066,8 +1081,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 	g_object_get (priv->calibrate_dialog, "reference-kind", &priv->reference_kind, NULL);
 
 	/* get default precision */
-	precision = gconf_client_get_string (priv->gconf_client, GCM_SETTINGS_CALIBRATION_LENGTH, NULL);
-	priv->precision = gcm_calibrate_precision_from_string (precision);
+	priv->precision = g_settings_get_enum (priv->settings, GCM_SETTINGS_CALIBRATION_LENGTH);
 	if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
 		priv->precision = gcm_calibrate_get_precision (calibrate, error);
 		if (priv->precision == GCM_CALIBRATE_PRECISION_UNKNOWN) {
@@ -1084,7 +1098,7 @@ gcm_calibrate_device (GcmCalibrate *calibrate, GtkWindow *window, GError **error
 		g_set_error_literal (error,
 				     GCM_CALIBRATE_ERROR,
 				     GCM_CALIBRATE_ERROR_USER_ABORT,
-				     "could not get reference image");
+				     "could not get calibration target image");
 		ret = FALSE;
 		goto out;
 	}
@@ -1205,6 +1219,9 @@ gcm_calibrate_get_property (GObject *object, guint prop_id, GValue *value, GPara
 	case PROP_PRECISION:
 		g_value_set_uint (value, priv->precision);
 		break;
+	case PROP_XYZ:
+		g_value_set_object (value, priv->xyz);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1295,6 +1312,11 @@ gcm_calibrate_set_property (GObject *object, guint prop_id, const GValue *value,
 		g_free (priv->working_path);
 		priv->working_path = g_strdup (g_value_get_string (value));
 		break;
+	case PROP_XYZ:
+		if (priv->xyz != NULL)
+			g_object_unref (priv->xyz);
+		priv->xyz = g_value_dup_object (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1449,6 +1471,14 @@ gcm_calibrate_class_init (GcmCalibrateClass *klass)
 				   G_PARAM_READABLE);
 	g_object_class_install_property (object_class, PROP_PRECISION, pspec);
 
+	/**
+	 * GcmCalibrate:xyz:
+	 */
+	pspec = g_param_spec_object ("xyz", NULL, NULL,
+				     GCM_TYPE_XYZ,
+				     G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_XYZ, pspec);
+
 	g_type_class_add_private (klass, sizeof (GcmCalibratePrivate));
 }
 
@@ -1469,6 +1499,7 @@ gcm_calibrate_init (GcmCalibrate *calibrate)
 	calibrate->priv->description = NULL;
 	calibrate->priv->device = NULL;
 	calibrate->priv->serial = NULL;
+	calibrate->priv->xyz = gcm_xyz_new ();
 	calibrate->priv->working_path = NULL;
 	calibrate->priv->calibrate_device_kind = GCM_CALIBRATE_DEVICE_KIND_UNKNOWN;
 	calibrate->priv->print_kind = GCM_CALIBRATE_PRINT_KIND_UNKNOWN;
@@ -1481,8 +1512,8 @@ gcm_calibrate_init (GcmCalibrate *calibrate)
 	// FIXME: this has to be per-run specific
 	calibrate->priv->working_path = g_strdup ("/tmp");
 
-	/* use GConf to get defaults */
-	calibrate->priv->gconf_client = gconf_client_get_default ();
+	/* use GSettings to get defaults */
+	calibrate->priv->settings = g_settings_new (GCM_SETTINGS_SCHEMA);
 
 	/* coldplug, and watch for changes */
 	calibrate->priv->colorimeter_kind = gcm_colorimeter_get_kind (calibrate->priv->colorimeter);
@@ -1510,10 +1541,12 @@ gcm_calibrate_finalize (GObject *object)
 	g_free (priv->serial);
 	g_free (priv->working_path);
 	g_signal_handlers_disconnect_by_func (calibrate->priv->colorimeter, G_CALLBACK (gcm_prefs_colorimeter_changed_cb), calibrate);
+	if (priv->xyz != NULL)
+		g_object_unref (priv->xyz);
 	g_object_unref (priv->colorimeter);
 	g_object_unref (priv->dmi);
 	g_object_unref (priv->calibrate_dialog);
-	g_object_unref (priv->gconf_client);
+	g_object_unref (priv->settings);
 
 	G_OBJECT_CLASS (gcm_calibrate_parent_class)->finalize (object);
 }
@@ -1531,45 +1564,3 @@ gcm_calibrate_new (void)
 	return GCM_CALIBRATE (calibrate);
 }
 
-/***************************************************************************
- ***                          MAKE CHECK TESTS                           ***
- ***************************************************************************/
-#ifdef EGG_TEST
-#include "egg-test.h"
-
-void
-gcm_calibrate_test (EggTest *test)
-{
-	GcmCalibrate *calibrate;
-	gboolean ret;
-	GError *error = NULL;
-	gchar *filename;
-
-	if (!egg_test_start (test, "GcmCalibrate"))
-		return;
-
-	/************************************************************/
-	egg_test_title (test, "get a calibrate object");
-	calibrate = gcm_calibrate_new ();
-	egg_test_assert (test, calibrate != NULL);
-
-	/************************************************************/
-	egg_test_title (test, "calibrate display manually");
-	filename = egg_test_get_data_file ("test.tif");
-	ret = gcm_calibrate_set_from_exif (GCM_CALIBRATE(calibrate), filename, &error);
-	if (!ret)
-		egg_test_failed (test, "error: %s", error->message);
-	else if (g_strcmp0 (gcm_calibrate_get_model_fallback (calibrate), "NIKON D60") != 0)
-		egg_test_failed (test, "got model: %s", gcm_calibrate_get_model_fallback (calibrate));
-	else if (g_strcmp0 (gcm_calibrate_get_manufacturer_fallback (calibrate), "NIKON CORPORATION") != 0)
-		egg_test_failed (test, "got manufacturer: %s", gcm_calibrate_get_manufacturer_fallback (calibrate));
-	else
-		egg_test_success (test, NULL);
-
-	g_object_unref (calibrate);
-	g_free (filename);
-
-	egg_test_end (test);
-}
-#endif
-
