@@ -34,13 +34,18 @@
 #include <gudev/gudev.h>
 #include <libgnomeui/gnome-rr.h>
 #include <cups/cups.h>
-#include <sane/sane.h>
+
+#ifdef HAVE_SANE
+ #include <sane/sane.h>
+#endif
 
 #include "gcm-client.h"
 #include "gcm-device-xrandr.h"
 #include "gcm-device-udev.h"
 #include "gcm-device-cups.h"
-#include "gcm-device-sane.h"
+#ifdef HAVE_SANE
+ #include "gcm-device-sane.h"
+#endif
 #include "gcm-device-virtual.h"
 #include "gcm-screen.h"
 #include "gcm-utils.h"
@@ -52,8 +57,11 @@ static void     gcm_client_finalize	(GObject     *object);
 #define GCM_CLIENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GCM_TYPE_CLIENT, GcmClientPrivate))
 
 static void gcm_client_xrandr_add (GcmClient *client, GnomeRROutput *output);
-static gboolean gcm_client_add_connected_devices_sane (GcmClient *client, GError **error);
-static gpointer gcm_client_add_connected_devices_sane_thrd (GcmClient *client);
+
+#ifdef HAVE_SANE
+static gboolean gcm_client_coldplug_devices_sane (GcmClient *client, GError **error);
+static gpointer gcm_client_coldplug_devices_sane_thrd (GcmClient *client);
+#endif
 
 /**
  * GcmClientPrivate:
@@ -65,6 +73,7 @@ struct _GcmClientPrivate
 	gchar				*display_name;
 	GPtrArray			*array;
 	GUdevClient			*gudev_client;
+	GSettings			*settings;
 	GcmScreen			*screen;
 	http_t				*http;
 	gboolean			 loading;
@@ -72,6 +81,8 @@ struct _GcmClientPrivate
 	gboolean			 use_threads;
 	gboolean			 init_cups;
 	gboolean			 init_sane;
+	guint				 refresh_id;
+	guint				 emit_added_id;
 };
 
 enum {
@@ -245,6 +256,51 @@ gcm_client_get_id_for_udev_device (GUdevDevice *udev_device)
 	return id;
 }
 
+/**
+ * gcm_client_remove_device_internal:
+ **/
+static gboolean
+gcm_client_remove_device_internal (GcmClient *client, GcmDevice *device, gboolean emit_signal, GError **error)
+{
+	gboolean ret = FALSE;
+	const gchar *device_id;
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+
+	g_return_val_if_fail (GCM_IS_CLIENT (client), FALSE);
+	g_return_val_if_fail (GCM_IS_DEVICE (device), FALSE);
+
+	/* lock */
+	g_static_mutex_lock (&mutex);
+
+	/* check device is not connected */
+	device_id = gcm_device_get_id (device);
+	if (gcm_device_get_connected (device)) {
+		g_set_error (error, 1, 0, "device %s is still connected", device_id);
+		goto out;
+	}
+
+	/* try to remove from array */
+	ret = g_ptr_array_remove (client->priv->array, device);
+	if (!ret) {
+		g_set_error_literal (error, 1, 0, "not found in device array");
+		goto out;
+	}
+
+	/* ensure signal handlers are disconnected */
+	g_signal_handlers_disconnect_by_func (device, G_CALLBACK (gcm_client_device_changed_cb), client);
+
+	/* emit a signal */
+	if (emit_signal) {
+		egg_debug ("emit removed: %s", device_id);
+		g_signal_emit (client, signals[SIGNAL_REMOVED], 0, device);
+	}
+out:
+	/* unlock */
+	g_static_mutex_unlock (&mutex);
+
+	return ret;
+}
+
 /**
  * gcm_client_gudev_remove:
  **/
@@ -252,6 +308,7 @@ static gboolean
 gcm_client_gudev_remove (GcmClient *client, GUdevDevice *udev_device)
 {
 	GcmDevice *device;
+	GError *error = NULL;
 	gchar *id;
 	gboolean ret = FALSE;
 
@@ -268,6 +325,16 @@ gcm_client_gudev_remove (GcmClient *client, GUdevDevice *udev_device)
 	/* set disconnected */
 	gcm_device_set_connected (device, FALSE);
 
+	/* remove device as it never had a profile assigned */
+	if (!gcm_device_get_saved (device)) {
+		ret = gcm_client_remove_device_internal (client, device, TRUE, &error);
+		if (!ret) {
+			egg_warning ("failed to remove: %s", error->message);
+			g_error_free (error);
+			goto out;
+		}
+	}
+
 	/* success */
 	ret = TRUE;
 out:
@@ -284,11 +351,9 @@ static gboolean
 gcm_client_gudev_add (GcmClient *client, GUdevDevice *udev_device)
 {
 	GcmDevice *device = NULL;
-	GcmDevice *device_tmp = NULL;
 	gboolean ret = FALSE;
 	const gchar *value;
 	GError *error = NULL;
-	GcmClientPrivate *priv = client->priv;
 
 	/* matches our udev rules, which we can change without recompiling */
 	value = g_udev_device_get_property (udev_device, "GCM_DEVICE");
@@ -304,40 +369,20 @@ gcm_client_gudev_add (GcmClient *client, GUdevDevice *udev_device)
 		goto out;
 	}
 
-	/* we might have a previous saved device with this ID, in which case nuke it */
-	device_tmp = gcm_client_get_device_by_id (client, gcm_device_get_id (device));
-	if (device_tmp != NULL) {
-		/* already disconnected device now connected */
-		g_object_unref (device);
-		device = g_object_ref (device_tmp);
-		gcm_device_set_connected (device, TRUE);
-	}
-
-	/* load the device */
-	ret = gcm_device_load (device, &error);
+	/* add device */
+	ret = gcm_client_add_device (client, device, &error);
 	if (!ret) {
-		egg_warning ("failed to load: %s", error->message);
+		egg_debug ("failed to set for device: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-
-	/* add to list */
-	g_ptr_array_add (priv->array, g_object_ref (device));
-
-	/* signal the addition */
-	egg_debug ("emit: added %s to device list", gcm_device_get_id (device));
-	g_signal_emit (client, signals[SIGNAL_ADDED], 0, device);
-
-	/* connect to the changed signal */
-	g_signal_connect (device, "changed", G_CALLBACK (gcm_client_device_changed_cb), client);
 out:
 	if (device != NULL)
 		g_object_unref (device);
-	if (device_tmp != NULL)
-		g_object_unref (device_tmp);
 	return ret;
 }
 
+#ifdef HAVE_SANE
 /**
  * gcm_client_sane_refresh_cb:
  **/
@@ -355,14 +400,14 @@ gcm_client_sane_refresh_cb (GcmClient *client)
 	/* rescan */
 	egg_debug ("rescanning sane");
 	if (client->priv->use_threads) {
-		thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_sane_thrd, client, FALSE, &error);
+		thread = g_thread_create ((GThreadFunc) gcm_client_coldplug_devices_sane_thrd, client, FALSE, &error);
 		if (thread == NULL) {
 			egg_debug ("failed to rescan sane devices: %s", error->message);
 			g_error_free (error);
 			goto out;
 		}
 	} else {
-		ret = gcm_client_add_connected_devices_sane (client, &error);
+		ret = gcm_client_coldplug_devices_sane (client, &error);
 		if (!ret) {
 			egg_debug ("failed to rescan sane devices: %s", error->message);
 			g_error_free (error);
@@ -372,6 +417,7 @@ gcm_client_sane_refresh_cb (GcmClient *client)
 out:
 	return FALSE;
 }
+#endif
 
 /**
  * gcm_client_uevent_cb:
@@ -380,20 +426,29 @@ static void
 gcm_client_uevent_cb (GUdevClient *gudev_client, const gchar *action, GUdevDevice *udev_device, GcmClient *client)
 {
 	gboolean ret;
+#ifdef HAVE_SANE
 	const gchar *value;
 	GcmDevice *device_tmp;
 	guint i;
+	gboolean enable;
 	GcmClientPrivate *priv = client->priv;
+#endif
 
 	if (g_strcmp0 (action, "remove") == 0) {
 		ret = gcm_client_gudev_remove (client, udev_device);
 		if (ret)
 			egg_debug ("removed %s", g_udev_device_get_sysfs_path (udev_device));
 
+#ifdef HAVE_SANE
 		/* we need to remove scanner devices */
 		value = g_udev_device_get_property (udev_device, "GCM_RESCAN");
 		if (g_strcmp0 (value, "scanner") == 0) {
 
+			/* are we ignoring scanners */
+			enable = g_settings_get_boolean (client->priv->settings, GCM_SETTINGS_ENABLE_SANE);
+			if (!enable)
+				return;
+
 			/* set all scanners as disconnected */
 			for (i=0; i<priv->array->len; i++) {
 				device_tmp = g_ptr_array_index (priv->array, i);
@@ -402,26 +457,50 @@ gcm_client_uevent_cb (GUdevClient *gudev_client, const gchar *action, GUdevDevic
 			}
 
 			/* find any others that might still be connected */
-			g_timeout_add (GCM_CLIENT_SANE_REMOVED_TIMEOUT, (GSourceFunc) gcm_client_sane_refresh_cb, client);
+			if (priv->refresh_id != 0)
+				g_source_remove (priv->refresh_id);
+			priv->refresh_id = g_timeout_add (GCM_CLIENT_SANE_REMOVED_TIMEOUT,
+							  (GSourceFunc) gcm_client_sane_refresh_cb,
+							  client);
+#if GLIB_CHECK_VERSION(2,25,8)
+			g_source_set_name_by_id (priv->refresh_id, "[GcmClient] refresh due to device remove");
+#endif
 		}
+#endif
 
 	} else if (g_strcmp0 (action, "add") == 0) {
 		ret = gcm_client_gudev_add (client, udev_device);
 		if (ret)
 			egg_debug ("added %s", g_udev_device_get_sysfs_path (udev_device));
 
+#ifdef HAVE_SANE
 		/* we need to rescan scanner devices */
 		value = g_udev_device_get_property (udev_device, "GCM_RESCAN");
-		if (g_strcmp0 (value, "scanner") == 0)
-			g_timeout_add (GCM_CLIENT_SANE_REMOVED_TIMEOUT, (GSourceFunc) gcm_client_sane_refresh_cb, client);
+		if (g_strcmp0 (value, "scanner") == 0) {
+
+			/* are we ignoring scanners */
+			enable = g_settings_get_boolean (client->priv->settings, GCM_SETTINGS_ENABLE_SANE);
+			if (!enable)
+				return;
+
+			if (priv->refresh_id != 0)
+				g_source_remove (priv->refresh_id);
+			priv->refresh_id = g_timeout_add (GCM_CLIENT_SANE_REMOVED_TIMEOUT,
+							  (GSourceFunc) gcm_client_sane_refresh_cb,
+							  client);
+#if GLIB_CHECK_VERSION(2,25,8)
+			g_source_set_name_by_id (priv->refresh_id, "[GcmClient] refresh due to device add");
+#endif
+		}
+#endif
 	}
 }
 
 /**
- * gcm_client_add_connected_devices_udev:
+ * gcm_client_coldplug_devices_udev:
  **/
 static gboolean
-gcm_client_add_connected_devices_udev (GcmClient *client, GError **error)
+gcm_client_coldplug_devices_udev (GcmClient *client, GError **error)
 {
 	GList *devices;
 	GList *l;
@@ -452,12 +531,12 @@ gcm_client_add_connected_devices_udev (GcmClient *client, GError **error)
 }
 
 /**
- * gcm_client_add_connected_devices_udev_thrd:
+ * gcm_client_coldplug_devices_udev_thrd:
  **/
 static gpointer
-gcm_client_add_connected_devices_udev_thrd (GcmClient *client)
+gcm_client_coldplug_devices_udev_thrd (GcmClient *client)
 {
-	gcm_client_add_connected_devices_udev (client, NULL);
+	gcm_client_coldplug_devices_udev (client, NULL);
 	return NULL;
 }
 
@@ -587,11 +666,8 @@ static void
 gcm_client_xrandr_add (GcmClient *client, GnomeRROutput *output)
 {
 	gboolean ret;
-	gboolean connected;
 	GError *error = NULL;
 	GcmDevice *device = NULL;
-	GcmDevice *device_tmp = NULL;
-	GcmClientPrivate *priv = client->priv;
 
 	/* if nothing connected then ignore */
 	ret = gnome_rr_output_is_connected (output);
@@ -609,51 +685,23 @@ gcm_client_xrandr_add (GcmClient *client, GnomeRROutput *output)
 		goto out;
 	}
 
-	/* we might have a previous saved device with this ID, in which use that instead */
-	device_tmp = gcm_client_get_device_by_id (client, gcm_device_get_id (device));
-	if (device_tmp != NULL) {
-
-		/* this is just a dupe */
-		connected = gcm_device_get_connected (device_tmp);
-		if (connected) {
-			egg_debug ("ignoring dupe");
-			goto out;
-		}
-
-		/* use original device */
-		g_object_unref (device);
-		device = g_object_ref (device_tmp);
-	}
-
-	/* load the device */
-	ret = gcm_device_load (device, &error);
+	/* add device */
+	ret = gcm_client_add_device (client, device, &error);
 	if (!ret) {
-		egg_warning ("failed to load: %s", error->message);
+		egg_debug ("failed to set for device: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-
-	/* add to the array */
-	g_ptr_array_add (priv->array, g_object_ref (device));
-
-	/* signal the addition */
-	egg_debug ("emit: added %s to device list", gcm_device_get_id (device));
-	g_signal_emit (client, signals[SIGNAL_ADDED], 0, device);
-
-	/* connect to the changed signal */
-	g_signal_connect (device, "changed", G_CALLBACK (gcm_client_device_changed_cb), client);
 out:
 	if (device != NULL)
 		g_object_unref (device);
-	if (device_tmp != NULL)
-		g_object_unref (device_tmp);
 }
 
 /**
- * gcm_client_add_connected_devices_xrandr:
+ * gcm_client_coldplug_devices_xrandr:
  **/
 static gboolean
-gcm_client_add_connected_devices_xrandr (GcmClient *client, GError **error)
+gcm_client_coldplug_devices_xrandr (GcmClient *client, GError **error)
 {
 	GnomeRROutput **outputs;
 	guint i;
@@ -691,33 +739,23 @@ gcm_client_cups_add (GcmClient *client, cups_dest_t dest)
 		goto out;
 	}
 
-	/* load the device */
-	ret = gcm_device_load (device, &error);
+	/* add device */
+	ret = gcm_client_add_device (client, device, &error);
 	if (!ret) {
-		egg_warning ("failed to load: %s", error->message);
+		egg_debug ("failed to set for device: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-
-	/* add to the array */
-	g_ptr_array_add (priv->array, g_object_ref (device));
-
-	/* signal the addition */
-	egg_debug ("emit: added %s to device list", gcm_device_get_id (device));
-	g_signal_emit (client, signals[SIGNAL_ADDED], 0, device);
-
-	/* connect to the changed signal */
-	g_signal_connect (device, "changed", G_CALLBACK (gcm_client_device_changed_cb), client);
 out:
 	if (device != NULL)
 		g_object_unref (device);
 }
 
 /**
- * gcm_client_add_connected_devices_cups:
+ * gcm_client_coldplug_devices_cups:
  **/
 static gboolean
-gcm_client_add_connected_devices_cups (GcmClient *client, GError **error)
+gcm_client_coldplug_devices_cups (GcmClient *client, GError **error)
 {
 	gint num_dests;
 	cups_dest_t *dests;
@@ -746,15 +784,16 @@ gcm_client_add_connected_devices_cups (GcmClient *client, GError **error)
 }
 
 /**
- * gcm_client_add_connected_devices_cups_thrd:
+ * gcm_client_coldplug_devices_cups_thrd:
  **/
 static gpointer
-gcm_client_add_connected_devices_cups_thrd (GcmClient *client)
+gcm_client_coldplug_devices_cups_thrd (GcmClient *client)
 {
-	gcm_client_add_connected_devices_cups (client, NULL);
+	gcm_client_coldplug_devices_cups (client, NULL);
 	return NULL;
 }
 
+#ifdef HAVE_SANE
 /**
  * gcm_client_sane_add:
  **/
@@ -764,8 +803,6 @@ gcm_client_sane_add (GcmClient *client, const SANE_Device *sane_device)
 	gboolean ret;
 	GError *error = NULL;
 	GcmDevice *device = NULL;
-	GcmDevice *device_tmp = NULL;
-	GcmClientPrivate *priv = client->priv;
 
 	/* create new device */
 	device = gcm_device_sane_new ();
@@ -776,43 +813,23 @@ gcm_client_sane_add (GcmClient *client, const SANE_Device *sane_device)
 		goto out;
 	}
 
-	/* we might have a previous saved device with this ID, in which use that instead */
-	device_tmp = gcm_client_get_device_by_id (client, gcm_device_get_id (device));
-	if (device_tmp != NULL) {
-		gcm_device_set_connected (device_tmp, TRUE);
-		g_object_unref (device);
-		device = g_object_ref (device_tmp);
-	}
-
-	/* load the device */
-	ret = gcm_device_load (device, &error);
+	/* add device */
+	ret = gcm_client_add_device (client, device, &error);
 	if (!ret) {
-		egg_warning ("failed to load: %s", error->message);
+		egg_debug ("failed to set for device: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-
-	/* add to the array */
-	g_ptr_array_add (priv->array, g_object_ref (device));
-
-	/* signal the addition */
-	egg_debug ("emit: added %s to device list", gcm_device_get_id (device));
-	g_signal_emit (client, signals[SIGNAL_ADDED], 0, device);
-
-	/* connect to the changed signal */
-	g_signal_connect (device, "changed", G_CALLBACK (gcm_client_device_changed_cb), client);
 out:
 	if (device != NULL)
 		g_object_unref (device);
-	if (device_tmp != NULL)
-		g_object_unref (device_tmp);
 }
 
 /**
- * gcm_client_add_connected_devices_sane:
+ * gcm_client_coldplug_devices_sane:
  **/
 static gboolean
-gcm_client_add_connected_devices_sane (GcmClient *client, GError **error)
+gcm_client_coldplug_devices_sane (GcmClient *client, GError **error)
 {
 	gint i;
 	gboolean ret = TRUE;
@@ -856,14 +873,15 @@ out:
 }
 
 /**
- * gcm_client_add_connected_devices_sane_thrd:
+ * gcm_client_coldplug_devices_sane_thrd:
  **/
 static gpointer
-gcm_client_add_connected_devices_sane_thrd (GcmClient *client)
+gcm_client_coldplug_devices_sane_thrd (GcmClient *client)
 {
-	gcm_client_add_connected_devices_sane (client, NULL);
+	gcm_client_coldplug_devices_sane (client, NULL);
 	return NULL;
 }
+#endif
 
 /**
  * gcm_client_add_unconnected_device:
@@ -880,7 +898,6 @@ gcm_client_add_unconnected_device (GcmClient *client, GKeyFile *keyfile, const g
 	gboolean ret;
 	gboolean virtual;
 	GError *error = NULL;
-	GcmClientPrivate *priv = client->priv;
 
 	/* add new device */
 	title = g_key_file_get_string (keyfile, id, "title", NULL);
@@ -911,8 +928,10 @@ gcm_client_add_unconnected_device (GcmClient *client, GKeyFile *keyfile, const g
 	} else if (kind == GCM_DEVICE_KIND_CAMERA) {
 		/* FIXME: use GPhoto? */
 		device = gcm_device_udev_new ();
+#ifdef HAVE_SANE
 	} else if (kind == GCM_DEVICE_KIND_SCANNER) {
 		device = gcm_device_sane_new ();
+#endif
 	} else {
 		egg_warning ("device kind internal error");
 		goto out;
@@ -929,23 +948,13 @@ gcm_client_add_unconnected_device (GcmClient *client, GKeyFile *keyfile, const g
 		      "colorspace", colorspace,
 		      NULL);
 
-	/* load the device */
-	ret = gcm_device_load (device, &error);
+	/* add device */
+	ret = gcm_client_add_device (client, device, &error);
 	if (!ret) {
-		egg_warning ("failed to load: %s", error->message);
+		egg_debug ("failed to set for device: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-
-	/* add to list */
-	g_ptr_array_add (priv->array, g_object_ref (device));
-
-	/* signal the addition */
-	egg_debug ("emit: added %s to device list", id);
-	g_signal_emit (client, signals[SIGNAL_ADDED], 0, device);
-
-	/* connect to the changed signal */
-	g_signal_connect (device, "changed", G_CALLBACK (gcm_client_device_changed_cb), client);
 out:
 	if (device != NULL)
 		g_object_unref (device);
@@ -954,10 +963,85 @@ out:
 	g_free (title);
 }
 
+/**
+ * gcm_client_possibly_migrate_config_file:
+ *
+ * Copy the configuration file from ~/.config/gnome-color-manager to ~/.config/color
+ **/
+static gboolean
+gcm_client_possibly_migrate_config_file (GcmClient *client)
+{
+	gchar *dest = NULL;
+	gchar *source = NULL;
+	GFile *gdest = NULL;
+	GFile *gsource = NULL;
+	gboolean ret = FALSE;
+	gint config_version;
+	GError *error = NULL;
+
+	/* have we already attempted this (check first to avoid stating a file */
+	config_version = g_settings_get_int (client->priv->settings,
+					     GCM_SETTINGS_MIGRATE_CONFIG_VERSION);
+	if (config_version >= GCM_CONFIG_VERSION_SHARED_SPEC)
+		goto out;
+
+	/* create default path */
+	source = g_build_filename (g_get_user_config_dir (),
+				   "gnome-color-manager",
+				   "device-profiles.conf",
+				   NULL);
+	gsource = g_file_new_for_path (source);
+
+	/* no old profile */
+	ret = g_file_query_exists (gsource, NULL);
+	if (!ret) {
+		g_settings_set_int (client->priv->settings,
+				    GCM_SETTINGS_MIGRATE_CONFIG_VERSION,
+				    GCM_CONFIG_VERSION_SHARED_SPEC);
+		goto out;
+	}
+
+	/* ensure new root exists */
+	dest = gcm_utils_get_default_config_location ();
+	ret = gcm_utils_mkdir_for_filename (dest, &error);
+	if (!ret) {
+		egg_warning ("failed to create new tree: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* copy to new root */
+	gdest = g_file_new_for_path (dest);
+	ret = g_file_copy (gsource, gdest, G_FILE_COPY_NONE, NULL, NULL, NULL, &error);
+	if (!ret) {
+		/* does the file already exist? -- i.e. the schema version was reset */
+		if (error->domain != G_IO_ERROR ||
+		    error->code != G_IO_ERROR_EXISTS) {
+			egg_warning ("failed to copy: %s", error->message);
+			g_error_free (error);
+			goto out;
+		}
+		g_error_free (error);
+	}
+
+	/* do not attempt to migrate this again */
+	g_settings_set_int (client->priv->settings,
+			    GCM_SETTINGS_MIGRATE_CONFIG_VERSION,
+			    GCM_CONFIG_VERSION_SHARED_SPEC);
+out:
+	g_free (source);
+	g_free (dest);
+	if (gsource != NULL)
+		g_object_unref (gsource);
+	if (gdest != NULL)
+		g_object_unref (gdest);
+	return ret;
+}
+
 /**
  * gcm_client_add_saved:
  **/
-gboolean
+static gboolean
 gcm_client_add_saved (GcmClient *client, GError **error)
 {
 	gboolean ret;
@@ -967,6 +1051,9 @@ gcm_client_add_saved (GcmClient *client, GError **error)
 	guint i;
 	GcmDevice *device;
 
+	/* copy from old location */
+	gcm_client_possibly_migrate_config_file (client);
+
 	/* get the config file */
 	filename = gcm_utils_get_default_config_location ();
 	egg_debug ("using %s", filename);
@@ -974,8 +1061,14 @@ gcm_client_add_saved (GcmClient *client, GError **error)
 	/* load the config file */
 	keyfile = g_key_file_new ();
 	ret = g_key_file_load_from_file (keyfile, filename, G_KEY_FILE_NONE, error);
-	if (!ret)
+	if (!ret) {
+		/* if the file is missing then ignore the error */
+		if ((*error)->domain == G_FILE_ERROR && (*error)->code == G_FILE_ERROR_NOENT) {
+			g_clear_error (error);
+			ret = TRUE;
+		}
 		goto out;
+	}
 
 	/* get groups */
 	groups = g_key_file_get_groups (keyfile, NULL);
@@ -997,6 +1090,8 @@ gcm_client_add_saved (GcmClient *client, GError **error)
 		}
 	}
 out:
+	/* inform the UI */
+	gcm_client_done_loading (client);
 	g_strfreev (groups);
 	g_free (filename);
 	g_key_file_free (keyfile);
@@ -1004,24 +1099,36 @@ out:
 }
 
 /**
- * gcm_client_add_connected:
+ * gcm_client_coldplug:
  **/
 gboolean
-gcm_client_add_connected (GcmClient *client, GcmClientColdplug coldplug, GError **error)
+gcm_client_coldplug (GcmClient *client, GcmClientColdplug coldplug, GError **error)
 {
-	gboolean ret;
+	gboolean ret = TRUE;
 	GThread *thread;
+	gboolean enable;
 
 	g_return_val_if_fail (GCM_IS_CLIENT (client), FALSE);
 
+	/* copy from old location */
+	gcm_client_possibly_migrate_config_file (client);
+
 	/* reset */
 	client->priv->loading_refcount = 0;
 
+	/* saved devices */
+	if (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_SAVED) {
+		gcm_client_add_loading (client);
+		ret = gcm_client_add_saved (client, error);
+		if (!ret)
+			goto out;
+	}
+
 	/* XRandR */
 	if (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_XRANDR) {
 		gcm_client_add_loading (client);
 		egg_debug ("adding devices of type XRandR");
-		ret = gcm_client_add_connected_devices_xrandr (client, error);
+		ret = gcm_client_coldplug_devices_xrandr (client, error);
 		if (!ret)
 			goto out;
 	}
@@ -1031,100 +1138,157 @@ gcm_client_add_connected (GcmClient *client, GcmClientColdplug coldplug, GError
 		gcm_client_add_loading (client);
 		egg_debug ("adding devices of type UDEV");
 		if (client->priv->use_threads) {
-			thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_udev_thrd, client, FALSE, error);
+			thread = g_thread_create ((GThreadFunc) gcm_client_coldplug_devices_udev_thrd, client, FALSE, error);
 			if (thread == NULL)
 				goto out;
 		} else {
-			ret = gcm_client_add_connected_devices_udev (client, error);
+			ret = gcm_client_coldplug_devices_udev (client, error);
 			if (!ret)
 				goto out;
 		}
 	}
 
 	/* CUPS */
-	if (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_CUPS) {
+	enable = g_settings_get_boolean (client->priv->settings, GCM_SETTINGS_ENABLE_CUPS);
+	if (enable && (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_CUPS)) {
 		gcm_client_add_loading (client);
 		egg_debug ("adding devices of type CUPS");
 		if (client->priv->use_threads) {
-			thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_cups_thrd, client, FALSE, error);
+			thread = g_thread_create ((GThreadFunc) gcm_client_coldplug_devices_cups_thrd, client, FALSE, error);
 			if (thread == NULL)
 				goto out;
 		} else {
-			ret = gcm_client_add_connected_devices_cups (client, error);
+			ret = gcm_client_coldplug_devices_cups (client, error);
 			if (!ret)
 				goto out;
 		}
 	}
 
+#ifdef HAVE_SANE
 	/* SANE */
-	if (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_SANE) {
+	enable = g_settings_get_boolean (client->priv->settings, GCM_SETTINGS_ENABLE_SANE);
+	if (enable && (!coldplug || coldplug & GCM_CLIENT_COLDPLUG_SANE)) {
 		gcm_client_add_loading (client);
 		egg_debug ("adding devices of type SANE");
 		if (client->priv->use_threads) {
-			thread = g_thread_create ((GThreadFunc) gcm_client_add_connected_devices_sane_thrd, client, FALSE, error);
+			thread = g_thread_create ((GThreadFunc) gcm_client_coldplug_devices_sane_thrd, client, FALSE, error);
 			if (thread == NULL)
 				goto out;
 		} else {
-			ret = gcm_client_add_connected_devices_sane (client, error);
+			ret = gcm_client_coldplug_devices_sane (client, error);
 			if (!ret)
 				goto out;
 		}
 	}
+#endif
 out:
 	return ret;
 }
 
+typedef struct {
+	GcmClient *client;
+	GcmDevice *device;
+} GcmClientEmitHelper;
+
 /**
- * gcm_client_add_virtual_device:
+ * gcm_client_emit_added_idle_cb:
+ **/
+static gboolean
+gcm_client_emit_added_idle_cb (GcmClientEmitHelper *helper)
+{
+	/* emit a signal */
+	egg_debug ("emit added: %s", gcm_device_get_id (helper->device));
+	g_signal_emit (helper->client, signals[SIGNAL_ADDED], 0, helper->device);
+	helper->client->priv->emit_added_id = 0;
+
+	g_object_unref (helper->client);
+	g_object_unref (helper->device);
+	g_free (helper);
+	return FALSE;
+}
+
+/**
+ * gcm_client_emit_added_idle:
+ **/
+static void
+gcm_client_emit_added_idle (GcmClient *client, GcmDevice *device)
+{
+	GcmClientEmitHelper *helper;
+	helper = g_new0 (GcmClientEmitHelper, 1);
+	helper->client = g_object_ref (client);
+	helper->device = g_object_ref (device);
+	client->priv->emit_added_id = g_idle_add ((GSourceFunc) gcm_client_emit_added_idle_cb, helper);
+#if GLIB_CHECK_VERSION(2,25,8)
+	g_source_set_name_by_id (client->priv->emit_added_id, "[GcmClient] emit added for device");
+#endif
+}
+
+/**
+ * gcm_client_add_device:
  **/
 gboolean
-gcm_client_add_virtual_device (GcmClient *client, GcmDevice *device, GError **error)
+gcm_client_add_device (GcmClient *client, GcmDevice *device, GError **error)
 {
 	gboolean ret = FALSE;
-	const gchar *id;
-	gboolean virtual;
+	const gchar *device_id;
 	GcmDevice *device_tmp = NULL;
+	GPtrArray *array;
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 
 	g_return_val_if_fail (GCM_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (GCM_IS_DEVICE (device), FALSE);
 
-	/* check removable */
-	virtual = gcm_device_get_virtual (device);
-	if (!virtual) {
-		g_set_error_literal (error, 1, 0, "cannot add non-virtual devices");
-		goto out;
-	}
+	/* lock */
+	g_static_mutex_lock (&mutex);
 
 	/* look to see if device already exists */
-	id = gcm_device_get_id (device);
-	device_tmp = gcm_client_get_device_by_id (client, id);
+	device_id = gcm_device_get_id (device);
+	device_tmp = gcm_client_get_device_by_id (client, device_id);
 	if (device_tmp != NULL) {
-		/* TRANSLATORS: error message */
-		g_set_error_literal (error, 1, 0, _("This device already exists"));
-		goto out;
+		egg_debug ("already exists, copy settings and remove old device: %s", device_id);
+		array = gcm_device_get_profiles (device_tmp);
+		if (array != NULL) {
+			gcm_device_set_profiles (device, array);
+			g_ptr_array_unref (array);
+		}
+		gcm_device_set_saved (device, gcm_device_get_saved (device_tmp));
+		ret = gcm_client_remove_device_internal (client, device_tmp, FALSE, error);
+		if (!ret)
+			goto out;
 	}
 
+	/* load the device */
+	ret = gcm_device_load (device, error);
+	if (!ret)
+		goto out;
+
 	/* add to the array */
 	g_ptr_array_add (client->priv->array, g_object_ref (device));
-
-	/* update status */
-	gcm_device_set_saved (device, FALSE);
-
-	/* emit a signal */
-	egg_debug ("emit added: %s", id);
-	g_signal_emit (client, signals[SIGNAL_ADDED], 0, device);
-
-	/* connect to the changed signal */
 	g_signal_connect (device, "changed", G_CALLBACK (gcm_client_device_changed_cb), client);
 
+	/* emit a signal from the main thread, not this one */
+	gcm_client_emit_added_idle (client, device);
+
 	/* all okay */
 	ret = TRUE;
 out:
+	/* unlock */
+	g_static_mutex_unlock (&mutex);
+
 	if (device_tmp != NULL)
 		g_object_unref (device_tmp);
 	return ret;
 }
 
+/**
+ * gcm_client_remove_device:
+ **/
+gboolean
+gcm_client_remove_device (GcmClient *client, GcmDevice *device, GError **error)
+{
+	return gcm_client_remove_device_internal (client, device, TRUE, error);
+}
+
 /**
  * gcm_client_delete_device:
  **/
@@ -1132,39 +1296,22 @@ gboolean
 gcm_client_delete_device (GcmClient *client, GcmDevice *device, GError **error)
 {
 	gboolean ret = FALSE;
-	const gchar *id;
+	const gchar *device_id;
 	gchar *data = NULL;
 	gchar *filename = NULL;
 	GKeyFile *keyfile = NULL;
-	gboolean saved;
-	gboolean connected;
 
 	g_return_val_if_fail (GCM_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (GCM_IS_DEVICE (device), FALSE);
 
-	/* check device is not connected */
-	connected = gcm_device_get_connected (device);
-	if (connected) {
-		g_set_error_literal (error, 1, 0, "device is still connected");
-		goto out;
-	}
-
-	/* try to remove from array */
-	ret = g_ptr_array_remove (client->priv->array, device);
-	if (!ret) {
-		g_set_error_literal (error, 1, 0, "not found in device array");
-		goto out;
-	}
-
 	/* check device is saved */
-	id = gcm_device_get_id (device);
-	saved = gcm_device_get_saved (device);
-	if (!saved)
-		goto not_saved;
+	device_id = gcm_device_get_id (device);
+	if (!gcm_device_get_saved (device))
+		goto out;
 
 	/* get the config file */
 	filename = gcm_utils_get_default_config_location ();
-	egg_debug ("removing %s from %s", id, filename);
+	egg_debug ("removing %s from %s", device_id, filename);
 
 	/* load the config file */
 	keyfile = g_key_file_new ();
@@ -1173,7 +1320,7 @@ gcm_client_delete_device (GcmClient *client, GcmDevice *device, GError **error)
 		goto out;
 
 	/* remove from the config file */
-	ret = g_key_file_remove_group (keyfile, id, error);
+	ret = g_key_file_remove_group (keyfile, device_id, error);
 	if (!ret)
 		goto out;
 
@@ -1192,10 +1339,10 @@ gcm_client_delete_device (GcmClient *client, GcmDevice *device, GError **error)
 	/* update status */
 	gcm_device_set_saved (device, FALSE);
 
-not_saved:
-	/* emit a signal */
-	egg_debug ("emit removed: %s", id);
-	g_signal_emit (client, signals[SIGNAL_REMOVED], 0, device);
+	/* remove device */
+	ret = gcm_client_remove_device_internal (client, device, TRUE, error);
+	if (!ret)
+		goto out;
 out:
 	g_free (data);
 	g_free (filename);
@@ -1352,6 +1499,7 @@ gcm_client_init (GcmClient *client)
 	client->priv->use_threads = FALSE;
 	client->priv->init_cups = FALSE;
 	client->priv->init_sane = FALSE;
+	client->priv->settings = g_settings_new (GCM_SETTINGS_SCHEMA);
 	client->priv->array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	client->priv->screen = gcm_screen_new ();
 	g_signal_connect (client->priv->screen, "outputs-changed",
@@ -1374,6 +1522,12 @@ gcm_client_finalize (GObject *object)
 	GcmDevice *device;
 	guint i;
 
+	/* disconnect anything that's about to fire */
+	if (priv->refresh_id != 0)
+		g_source_remove (priv->refresh_id);
+	if (priv->emit_added_id != 0)
+		g_source_remove (priv->emit_added_id);
+
 	/* do not respond to changed events */
 	for (i=0; i<priv->array->len; i++) {
 		device = g_ptr_array_index (priv->array, i);
@@ -1384,10 +1538,13 @@ gcm_client_finalize (GObject *object)
 	g_ptr_array_unref (priv->array);
 	g_object_unref (priv->gudev_client);
 	g_object_unref (priv->screen);
+	g_object_unref (priv->settings);
 	if (client->priv->init_cups)
 		httpClose (priv->http);
+#ifdef HAVE_SANE
 	if (client->priv->init_sane)
 		sane_exit ();
+#endif
 
 	G_OBJECT_CLASS (gcm_client_parent_class)->finalize (object);
 }
